function data=shAccuracyTest(L,os_factor,range,save_path,method)
 % data = shAccuracyTest(L,os_factor,range,save_path);
 %
 % Example: shAccuracyTest(10,1.5,[1:50],'results');
 %
 % Compute random rotations and compare error to analytic spherical bessel
 % functions.  A number of quasi-random rotations are performed, and 
 % L^2, L^1 and L^inf error is stored for each.
 %
 % The sequence of rotations performed is quasi-random and deterministic,
 % so a range is specified.  Calling function twice with same range will
 % just repeat previous rotations.
 %
 % @return data         results matrix, each row corresponds to a result
 % for a SH band.
 % format:
 % [band] [cond. num] [max L^2 error] [angle1] [angle2] [angle3] [num]
 % @param  L            maximum SH band
 % @param  os_factor    oversampling factor
 % @param  range        range of random rotations to perform
 % @param  savepath     directory to store results
  
    defval('num',1)
    defval('save_path','results')
    defval('method','halton')
    filename=[save_path,'/accuracy_',num2str(L),'_',num2str(os_factor*100)];


    % load previous data if it exists
    if (exist([filename,'.mat'],'file'))
      load(filename);
    else
      data=[];
    end
        
    N = 2 * L + 1;
    num_samples = ceil(N);

    % Seed pseudo-random generator
    rand('twister',5489)   
    axis_rand = rand(3,1000);
    angle_rand = halton(1000,1) * 360;
    
    %rand('twister',sum(100*clock))    
    
    % calculate benchmark SH coefs
    % location of accoustic source
    %  source_loc = randomPointsS2( rand(1, 2))
    
    %  source_loc = randomPointsS2( rand(1, 2))
    
    source_loc = deg_to_rad([90 45])
    
    % calculate SH coeffs for accoustic wave
    coeffs=[];
    q=shCompleteBasisL(L,source_loc(1),source_loc(2));
    for l=0:L
        coeffs = vertcat(coeffs, q{l+1});
    end
        
    % normalize magnitude of coefficients within each band
    % so that we can compare error between bands.
    for i=0:L
        index = i^2 + 1;
        coeffs(index:index+2*i) = coeffs(index:index+2*i)/ norm(coeffs(index:index+2*i));
    end

    clear q;
    disp('done source coeffs');
    
    for zz=range
      
      axis = axis_rand(:,zz);
      axis = (axis - 0.5) * 2.0;
      axis = axis / norm(axis);

      angle = angle_rand(zz);

      % Rotate location of accoustic source
      R_source = getRotationMatrix( axis, -angle);

      p_c = s2_to_r3( source_loc);
      p_c = R_source * p_c;
      source_loc_r = r3_to_s2( p_c);

      % Calculate SH coeffs for rotated accoustic wave.

      sh_bessel=[];
      q=shCompleteBasisL(L,source_loc_r(1),source_loc_r(2));
      for l=0:L
          sh_bessel = vertcat(sh_bessel, q{l+1});
      end

      % normalize magnitude of coefficients within each band
      % so that we can compare error between bands.
      for i=0:L
        index = i^2 + 1;
        sh_bessel(index:index+2*i) = sh_bessel(index:index+2*i)/ norm(sh_bessel(index:index+2*i));
      end

      clear q;
      disp('done rotated coefs');

      %sh_bessel2=sh_bessel((size(sh_bessel,1)-N)+1:size(sh_bessel,1));
      %sh_bessel2=sh_bessel;

      %banded_error = norm(sh_method2 - sh_bessel2)

      [sh_method,cond_nums] = shRKHSRotation(L,coeffs,os_factor,axis,angle,save_path,method);

      sh_error = norm(sh_method - sh_bessel);
      edata = sh_method - sh_bessel;        
      for i=0:L
        index = i^2 + 1;
        l2_error = norm(edata(index: index+2*i));
        l1_error = norm(edata(index: index+2*i),1);
        linf_error = norm(edata(index: index+2*i),inf);
        data((zz-1) * (L+1) + i+1, :) = [i cond_nums(i+1) l2_error l1_error linf_error axis(1) axis(2) axis(3) angle];
      end

      % Save data
      % format: accuracy_[max band]_[oversampling factor*100]

      filename=[save_path,'/accuracy_',num2str(L),'_',num2str(os_factor*100)];
      save(filename,'data');
    
    end

    % plot data
    % shPlotAccuracy(L,os_factor,save_path);

    % maximum error
    max_error=max(data(:,3))
end
