function f = shEvalFunction( coeffs, num_bands, omega) 
%
% f = shEvalFunction( coeffs, num_bands, omega) 
%
% Evaluate the function f represented by \a coeffs at the point 
% omega = (theta,phi). The function f can have different channels which
% have to correspond to different rows of \a coeffs. Multiple rows of omega
% correspond to multiple points where f is to be evaluated.
%
% @param  coeffs  Spherical Harmonics representation of the function f to
%                 evaluate
% @param  num_bands  Number of Spherical Harmonics bands used for the
%                    representation
% @param  omega  Point(s) on S^2 where to evaluate f

  num_points = size( omega,1);
  num_channels = size( coeffs, 1);
  f = zeros( num_channels, num_points);

  for( l = 0 : (num_bands - 1))
    for( m = -l : l)
      index = l * (l+1) + m + 1;
      f = f + repmat( coeffs(:,index), 1, num_points) ...
             .* repmat( shEval( l, m, omega(:,1), omega(:,2)), num_channels, 1); 
    end
  end
  
end