function [p3] = shLegendreRecurrenceNorm(L,cosx,sinx,p2)
  % 

  % p3 = legendreRecurranceNorm(L,cosx,sinx,p2)
  %
  % Calculates legendre polynomials of band L for value cosx, given
  % bands L-1 
  % Uses Schmidt semi-normalization. 
  % See 'help legendre' for more info about Schmidt semi-normalization
  %
  % @return p3    legendre values for band L
  % @param L      SH band we wish to calculate
  % @param cosx   precomputed cos(x)
  % @param sinx   precomputed sin(x)
  % @param p2     legendre values for band L - 1
  
  defval('p2',[]);
  defval('p1',[]);
  L_index = L+1;
  p3=zeros(L,1);
  
  % Initialization
  if (L==0) 
      p3 = [1];
      return;
  end
    
  % for l > 1, use previous bands for recurrence
  % first recurrence rule
  m=L;
  m_index=m+1;
  if (m==1)
    % no normalization factor
    scale2=-1;
  else
    scale2 = - sqrt(1/( (L+m) * (L+m-1)));
  end
  
  p3(m_index) = -(2*m - 1) * scale2 * p2(m_index-1);


  % second recurrence rule 

  m=L-1;
  m_index=m+1;
  scale1=1;
  if (m==0)
      scale1 = sqrt( ((L+1) * L) / 2)   / ((L+(m+1)) * (L-(m+1)+1));
  else
      scale1  = 1/(sqrt( (L-m) * (L+m+1)));
  end

  p3(m_index) =2*(m+1)*(cosx) * scale1 * p3(m_index+1);
  
  if (L==1) return; end;
      
  % 3rd reccurance rule
  for m=L-2:-1:0
      m_index=m+1;
      scale1=1;
      scale2=1;
      if (m==0)
          scale1 = sqrt( ((L+1) * L) / 2)   / ((L+(m+1)) * (L-(m+1)+1));
          scale2 = sqrt( ((L+2) * (L+1) * L * (L-1)) / 2) / ((L+(m+1)) * (L-(m+1)+1));
      else
         scale1  = 1/(sqrt( (L-m) * (L+m+1)));
         scale2  = sqrt( ((L-m-1) * (L+m+2)) / ((L-m) * (L+m+1)));
      end

      p3(m_index) =(2*(m+1)*(cosx) * scale1 * p3(m_index+1) - scale2  * sinx * sinx* p3(m_index+2));
      
  end
  
end
