function coeffs = shProjectPhong( samples, max_band, reflect_vector, exponent)
%
% coeffs = shProjectPhong( samples, num_bands, exponent)
%
% Project scaled local cosine term into Spherical Harmonics with max_bands
% bands.

  coeffs = zeros( 1, (max_band+1)^2);
  
  % do for all samples
  for( i = 1 : numel( samples))
    
    sample = samples{i};
    
    % compute scaled cosine term, restricted to the upper hemisphere
    val = max( 0, dot( reflect_vector, sample.vec)^exponent);
    
    % replicate for vectorized processing
    signal_sample = repmat( val, 1, numel( sample.coeffs));

    % project for all bands simultaneously
    coeffs = coeffs + (sample.coeffs .* signal_sample);
    
  end

  % normalize
  factor = (4 * pi) / numel( samples);
  coeffs = coeffs * factor;
  
end
